/****************************************************************************
    
    jackpluginwrapper.hpp - a class that should make it easier to write 
                            DSSI hosts in C++
    
    Copyright (C) 2005  Lars Luthman <larsl@users.sourceforge.net>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 01222-1307  USA

****************************************************************************/

#ifndef JACKPLUGINWRAPPER_HPP
#define JACKPLUGINWRAPPER_HPP

#include <jack/jack.h>
#include <jack/midiport.h>
#include <alsa/asoundlib.h>

#include "pluginwrapper.hpp"


/** This class extends PluginWrapper by adding a set_client() function
    that can be used to let the plugin know which JACK client to use.
    If set_client() is called before activate(), JACK audio and MIDI ports
    will be created and connected to the plugin ports next time activate()
    is called. If set_client() is not used this class works just like
    PluginWrapper. */
class JACKPluginWrapper : public PluginWrapper {
public:
  
  JACKPluginWrapper(const std::string& identifier);
  ~JACKPluginWrapper();
  
  void set_client(jack_client_t* client);
  void activate();
  void run(unsigned long frames);
  
protected:
  
  struct PortMap {
    PortMap(jack_port_t* jp, unsigned long pp) : jack(jp), plugin(pp) { }
    jack_port_t* jack;
    unsigned long plugin;
  };
  
  jack_client_t* m_client;
  std::vector<PortMap> m_jack_input_ports;
  std::vector<PortMap> m_jack_output_ports;
  jack_port_t* m_midi_input_port;
  std::vector<LADSPA_Data*> m_controls;
  bool m_program_selected;
  snd_seq_event_t m_midi_events[1000];
  snd_midi_event_t *m_alsa_encoder;
  unsigned int m_current_bank;

};


#endif
