/****************************************************************************
    
    simplehost.cpp - A very simple and incomplete DSSI host
    
    Copyright (C) 2005  Lars Luthman <larsl@users.sourceforge.net>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 01222-1307  USA

****************************************************************************/

#include <csignal>
#include <iostream>
#include <map>
#include <vector>
#include <utility>

#include <jack/jack.h>
#include <jack/midiport.h>

#include "jackpluginwrapper.hpp"


using namespace std;


static JACKPluginWrapper* plugin = NULL;
static bool running = true;


/** Signal handler that allows us to clean up if the user terminates 
    the program. */
void signal_handler(int sig) {
  running = false;
}


int process(jack_nframes_t frames, void* arg);


int main(int argc, char** argv) {
  
  // register our signal handler
  signal(SIGINT, &signal_handler);
  signal(SIGTERM, &signal_handler);

  // we need at least one parameter
  if (argc <= 1) {
    cerr<<"You need to specify a plugin like this: "
	<<"'simplehost DSSI:sineshaper.so:ll-sineshaper'"<<endl;
    return 1;
  }
  
  // load the plugin
  try {
    plugin = new JACKPluginWrapper(argv[1]);
  }
  catch (const exception& e) {
    cerr<<"Could not load the plugin "<<argv[1]<<". Exiting."<<endl;
    return 1;
  }
  
  // print some info about the plugin
  cout<<endl<<plugin->name()<<" by "<<plugin->maker()<<endl
      <<"Copyright: "<<plugin->copyright()<<endl<<endl;
  
  // create the JACK client
  jack_client_t* client = jack_client_new("Simple DSSI host");
  jack_set_process_callback(client, &process, NULL);
  
  // activate everything
  plugin->set_client(client);
  plugin->activate();
  jack_activate(client);
  
  while (running) sleep(666);
  
  // clean up
  jack_deactivate(client);
  jack_client_close(client);
  plugin->deactivate();
  delete plugin;

  return 0;
}


int process(jack_nframes_t frames, void* arg) {
  plugin->run(frames);
  return 0;
}

